from collections import OrderedDict

from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil


class MetaData:
    def __init__(self, input_data_org):
        # Common Param
        self.transaction_id = PhUtil.generate_transaction_id()
        self.input_data_org = input_data_org
        self.parsed_data = None
        self.operation_mode = None
        self.input_mode_key = None
        self.output_file_path = None
        self.include_files = None
        self.excludes = None
        self.output_dic = OrderedDict()
        self.input_mode_value = str(self.input_data_org)
        #
        self.output_file_ext_default = None
        self.output_file_location_default = None

    def get_info_data(self):
        if self.output_dic:
            info = self.output_dic.get(PhKeys.INFO, None)
            if info is not None:
                return str(info)
        return PhConstants.STR_EMPTY

    def get_parsed_data(self):
        return self.parsed_data



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil
from play_helpers.ph_variables import PhVariables


class Data:
    def __init__(self,
                 # Common Param
                 input_data=None,
                 print_input=None,
                 print_output=None,
                 print_info=None,
                 quite_mode=None,
                 remarks=[],
                 encoding=None,
                 encoding_errors=None,
                 output_path=None,
                 output_file_name_keyword=None,
                 archive_output=None,
                 archive_output_format=None,
                 # Specific Param
                 length_in_decimal=None,
                 value_in_ascii=None,
                 one_liner=None,
                 non_tlv_neighbor=None,
                 # Unknown/System Param
                 **kwargs,
                 ):
        """
        Instantiate the Data Object for further Processing.

        :param input_data: Input Data; String(s), File Path(s), Dir Paths(s)
        :param print_input: Printing of input needed?
        :param print_output: Printing of output needed?
        :param print_info:  Printing of info needed?
        :param quite_mode: Quite mode needed? If yes, no printing at all.
        :param remarks: Remarks for Input Data
        :param encoding: Encoding for Input/Output Data
        :param encoding_errors: Encoding Errors Handling for Input/Output Data
        :param output_path: Output Path
        :param output_file_name_keyword:
        :param archive_output: Archiving of output needed?
        :param archive_output_format: Archive Output Format
        :param length_in_decimal: Length in Decimal is needed ?
        :param value_in_ascii: Value in ASCII is needed ?
        :param one_liner: One Liner output needed ?
        :param non_tlv_neighbor: Non TLV Neighbor needs to be handled ?
        :param kwargs: To Handle unwanted/deprecated/internal/additional arguments (See Description)
        ----------

        kwargs -- (handled arguments description)
            raw_data -- @Deprecated!!! Use input_data instead \n
            remarks_list -- @Deprecated!!! Use remarks instead \n
            data_group -- Used for Web App
        ----------
        """
        # Handle Normal Args
        self.input_data = input_data
        self.print_input = print_input
        self.print_output = print_output
        self.print_info = print_info
        self.quite_mode = quite_mode
        self.remarks = remarks
        self.encoding = encoding
        self.encoding_errors = encoding_errors
        self.output_path = output_path
        self.output_file_name_keyword = output_file_name_keyword
        self.archive_output = archive_output
        self.archive_output_format = archive_output_format
        self.length_in_decimal = length_in_decimal
        self.value_in_ascii = value_in_ascii
        self.one_liner = one_liner
        self.non_tlv_neighbor = non_tlv_neighbor
        # Handle kwargs
        if self.input_data is None and PhKeys.RAW_DATA in kwargs:
            self.input_data = kwargs[PhKeys.RAW_DATA]
        if self.remarks is None and PhKeys.REMARKS_LIST in kwargs:
            self.remarks = kwargs[PhKeys.REMARKS_LIST]
        self.data_group = kwargs.get(PhKeys.DATA_GROUP, None)
        # Handle Internal args
        self.__input_modes_hierarchy = []
        self.__auto_generated_remarks = None
        self.__one_time_remarks = None
        self.__extended_remarks_needed = None
        # Handle Remarks
        self.set_user_remarks(self.remarks)
        # Handle Remarks Variables
        self.__variables_pool = {
            #           _key : (_var_name, _var_value)
            PhKeys.LENGTH_IN_DECIMAL: (PhVariables.LENGTH_IN_DECIMAL, self.length_in_decimal),
            PhKeys.VALUE_IN_ASCII: (PhVariables.VALUE_IN_ASCII, self.value_in_ascii),
            PhKeys.ONE_LINER: (PhVariables.ONE_LINER, self.one_liner),
            PhKeys.NON_TLV_NEIGHBOR: (PhVariables.NON_TLV_NEIGHBOR, self.non_tlv_neighbor),
        }

    def set_user_remarks(self, remarks):
        self.remarks = PhUtil.to_list(remarks, trim_data=True, all_str=True)

    def set_user_remarks_expand_variables(self):
        def __set_value(x, var_name, var_value, key_name_needed, key_):
            if var_name in x and var_value is not None:
                var_value = str(var_value)
                y = '_'.join([key_, var_value]) if key_name_needed else var_value
                return x.replace(var_name, y)
            return x

        def __expand_variable(x):
            key_name_needed = True if PhVariables.KEY_NAME in x else False
            if key_name_needed:
                x = x.replace(PhVariables.KEY_NAME, '')
            for key, value in self.__variables_pool.items():
                x = __set_value(x=x, var_name=value[0], var_value=value[1], key_name_needed=key_name_needed, key_=key)
            return x

        self.remarks = [__expand_variable(x) for x in self.remarks]

    def __get_default_remarks(self):
        str_input_data = PhUtil.combine_list_items(self.input_data)
        return str_input_data

    def reset_auto_generated_remarks(self):
        self.__auto_generated_remarks = None

    def set_auto_generated_remarks_if_needed(self, internal_remarks=None):
        internal_remarks = PhUtil.set_if_none(internal_remarks)
        default_remarks = self.__get_default_remarks()
        if self.remarks and self.remarks[0]:
            # User Remarks is already provided, default remarks are not needed
            default_remarks = None
        # auto generated comments are set
        self.__auto_generated_remarks = PhUtil.append_remarks(internal_remarks,
                                                              self.__auto_generated_remarks if self.__auto_generated_remarks else default_remarks,
                                                              append_mode_post=False)

    def get_remarks_as_str(self, user_original_remarks=False, force_mode=False):
        user_remarks = PhUtil.combine_list_items(self.remarks)
        if user_original_remarks:
            if user_remarks:
                return user_remarks.replace('\n', ' ')
            if not force_mode:
                return ''
        if user_remarks:
            user_remarks = PhUtil.trim_remarks(user_remarks)
        one_time_remarks = PhUtil.append_remarks(self.get_one_time_remarks(), self.__auto_generated_remarks,
                                                 append_mode_post=False)
        return PhUtil.append_remarks(one_time_remarks, user_remarks, append_mode_post=False).replace('\n', ' ')

    def set_extended_remarks_available(self, extended_remarks):
        self.__extended_remarks_needed = extended_remarks

    def get_extended_remarks_available(self):
        return self.__extended_remarks_needed

    def get_one_time_remarks(self):
        temp = self.__one_time_remarks
        self.__one_time_remarks = None
        return temp

    def set_one_time_remarks(self, one_time_remarks):
        self.__one_time_remarks = one_time_remarks

    def append_input_modes_hierarchy(self, input_mode_hierarchy):
        self.__input_modes_hierarchy.append(input_mode_hierarchy)

    def get_input_modes_hierarchy_as_str(self):
        return PhConstants.SEPERATOR_MULTI_OBJ.join(self.__input_modes_hierarchy)

    def get_input_modes_hierarchy(self):
        return self.__input_modes_hierarchy

    def validate_if_input_modes_hierarchy(self, input_mode_hierarchy):
        return True if input_mode_hierarchy in self.__input_modes_hierarchy else False



# from tlv_play._git_info import GIT_SUMMARY
from tlv_play._tool_name import TOOL_NAME, TOOL_TITLE, TOOL_PACKAGE_NAME, TOOL_TEST_PACKAGE_NAME

_run_time = False
try:
    from tlv_play._version import __version__

    # this is available post installation, run time only,
    _run_time = True
except ImportError:
    pass


class ConfigConst:
    # TODO:
    TOOL_VERSION = __version__.public() if _run_time else '1.0.0' #PhDefaults.VERSION
    TOOL_VERSION_DETAILED = f'v{TOOL_VERSION}'
    TOOL_NAME = TOOL_NAME
    TOOL_TITLE = TOOL_TITLE
    TOOL_PACKAGE_NAME = TOOL_PACKAGE_NAME
    TOOL_TEST_PACKAGE_NAME = TOOL_TEST_PACKAGE_NAME
    # TODO:
    # TOOL_GIT_SUMMARY = GIT_SUMMARY
    TOOL_GIT_SUMMARY = ''
    TOOL_DESCRIPTION = f'Generic TLV Parser based on TLV and related standards. Can parse any Simple or BER TLV upto *nth Level* automatically. Multi byte Tag, Multi byte Length are also Supported.'
    TOOL_META_DESCRIPTION = f'{TOOL_DESCRIPTION}'
    TOOL_META_KEYWORDS = f'{TOOL_TITLE}, TLV, Tag Length Value, Type Length Value, length value, tlv encoder, tlv decoder, tlv parser, multi byte tag, multi byte length, simple tlv, ber tlv, tlv utilities'
    TOOL_URL = 'https://github.com/impratikjaiswal/tlvPlay'
    TOOL_URL_BUG_TRACKER = 'https://github.com/impratikjaiswal/tlvPlay/issues'



from tlv_play.main.data_type.data_type_master import DataTypeMaster


class Dev(DataTypeMaster):

    def __init__(self):
        super().__init__()

    def set_print_input(self):
        print_input = None
        super().set_print_input(print_input)

    def set_print_output(self):
        print_output = None
        super().set_print_output(print_output)

    def set_print_info(self):
        print_info = None
        super().set_print_info(print_info)

    def set_quiet_mode(self):
        quite_mode = None
        super().set_quiet_mode(quite_mode)

    def set_remarks(self):
        remarks = None
        super().set_remarks(remarks)

    def set_encoding(self):
        encoding = None
        super().set_encoding(encoding)

    def set_encoding_errors(self):
        encoding_errors = None
        super().set_encoding_errors(encoding_errors)

    def set_output_path(self):
        output_path = None
        super().set_output_path(output_path)

    def set_output_file_name_keyword(self):
        output_file_name_keyword = None
        super().set_output_file_name_keyword(output_file_name_keyword)

    def set_archive_output(self):
        archive_output = None
        super().set_archive_output(archive_output)

    def set_archive_output_format(self):
        archive_output_format = None
        super().set_archive_output_format(archive_output_format)

    def set_length_in_decimal(self):
        length_in_decimal = None
        super().set_length_in_decimal(length_in_decimal)

    def set_value_in_ascii(self):
        value_in_ascii = None
        super().set_value_in_ascii(value_in_ascii)

    def set_one_liner(self):
        one_liner = None
        super().set_one_liner(one_liner)

    def set_non_tlv_neighbor(self):
        non_tlv_neighbor = None
        super().set_non_tlv_neighbor(non_tlv_neighbor)

    def set_data_pool(self):
        data_pool = [
            #
        ]
        super().set_data_pool(data_pool)



import io
import unittest.mock

from play_helpers.ph_keys import PhKeys

import test_tlv_data
from tlv_play.main.convert.converter import set_defaults
from tlv_play.main.helper.data import Data
from tlv_play.main.tlv.tlv_handler import TlvHandler
from tlv_play.main.tlv.tlv_parser import TlvParser

STR_TEST_OBJ = 'test_obj :'


class TestObjTlv:
    def __init__(self, input_data, expected_op, expected_op_print, test_name='', length_in_decimal=None,
                 value_in_ascii=None, one_liner=None, non_tlv_neighbor=None):
        data = Data(length_in_decimal=length_in_decimal, value_in_ascii=value_in_ascii, one_liner=one_liner,
                    non_tlv_neighbor=non_tlv_neighbor)
        set_defaults(data, None)
        self.input_data = input_data
        self.expected_op = expected_op
        self.expected_op_print = expected_op_print
        self.test_name = test_name
        self.length_in_decimal = data.length_in_decimal
        self.value_in_ascii = data.value_in_ascii
        self.one_liner = data.one_liner
        self.non_tlv_neighbor = data.non_tlv_neighbor


class TestTlv(unittest.TestCase):
    test_obj_pool = [
        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NON_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NON_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NON_NESTED_TLV,
                   test_name='Single Non-Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTIPLE_NON_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_MULTIPLE_NON_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_MULTIPLE_NON_NESTED_TLV,
                   test_name='Multiple Non-Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_POSSIBLE_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_POSSIBLE_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_POSSIBLE_NESTED_TLV,
                   test_name='Single Possible Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_THREE_BYTES_TAG_NO_LENGTH,
                   expected_op=test_tlv_data.LIST_OP_THREE_BYTES_TAG_NO_LENGTH,
                   expected_op_print=test_tlv_data.STR_OP_THREE_BYTES_TAG_NO_LENGTH,
                   test_name='Three Bytes Tag, No Length',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_FOUR_BYTES_TAG_NO_LENGTH,
                   expected_op=test_tlv_data.LIST_OP_FOUR_BYTES_TAG_NO_LENGTH,
                   expected_op_print=test_tlv_data.STR_OP_FOUR_BYTES_TAG_NO_LENGTH,
                   test_name='Four Bytes Tag, No Length',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_THREE_BYTES_TAG_0_LENGTH,
                   expected_op=test_tlv_data.LIST_OP_THREE_BYTES_TAG_0_LENGTH,
                   expected_op_print=test_tlv_data.STR_OP_THREE_BYTES_TAG_0_LENGTH,
                   test_name='Three Bytes Tag, 0 Length',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_FOUR_BYTES_TAG_0_LENGTH,
                   expected_op=test_tlv_data.LIST_OP_FOUR_BYTES_TAG_0_LENGTH,
                   expected_op_print=test_tlv_data.STR_OP_FOUR_BYTES_TAG_0_LENGTH,
                   test_name='Four Bytes Tag, 0 Length',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTIPLE_NON_NESTED_AND_SINGLE_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_MULTIPLE_NON_NESTED_AND_SINGLE_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_MULTIPLE_NON_NESTED_AND_SINGLE_NESTED_TLV,
                   test_name='Multiple Non Nested & Single Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTIPLE_NON_NESTED_AND_MULTIPLE_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_MULTIPLE_NON_NESTED_AND_MULTIPLE_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_MULTIPLE_NON_NESTED_AND_MULTIPLE_NESTED_TLV,
                   test_name='Multiple Non Nested & Multiple Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NESTED_TLV_WITH_1_LEN,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NESTED_TLV_WITH_1_LEN,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NESTED_TLV_WITH_1_LEN,
                   test_name='Single Nested TLV with 1 len',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NESTED_TLV_WITH_2_LEN,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NESTED_TLV_WITH_2_LEN,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NESTED_TLV_WITH_2_LEN,
                   test_name='Single Nested TLV with 2 len',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NESTED_TLV_POSSIBLE_SUB_NESTED,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NESTED_TLV_POSSIBLE_SUB_NESTED,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NESTED_TLV_POSSIBLE_SUB_NESTED,
                   test_name='Single Nested TLV, Possible Sub Nested',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTIPLE_SUB_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_MULTIPLE_SUB_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_MULTIPLE_SUB_NESTED_TLV,
                   test_name='Multiple Sub Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTIPLE_SUB_NESTED_TLV_POSSIBLE_GRAND_SUB_NESTED,
                   expected_op=test_tlv_data.LIST_OP_MULTIPLE_SUB_NESTED_TLV_POSSIBLE_GRAND_SUB_NESTED,
                   expected_op_print=test_tlv_data.STR_OP_MULTIPLE_SUB_NESTED_TLV_POSSIBLE_GRAND_SUB_NESTED,
                   test_name='Multiple Sub Nested TLV, Possible Grand Sub Nested',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTI_BYTE_TAG_FAKE_SUB_NESTED,
                   expected_op=test_tlv_data.LIST_OP_MULTI_BYTE_TAG_FAKE_SUB_NESTED,
                   expected_op_print=test_tlv_data.STR_OP_MULTI_BYTE_TAG_FAKE_SUB_NESTED,
                   test_name='Multi Byte Tag, Fake Sub Nested',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_MULTI_BYTE_TAG_FAKE_SUB_NESTED_ALONG_WITH_SIBLINGS_TLV,
                   expected_op=test_tlv_data.LIST_OP_MULTI_BYTE_TAG_FAKE_SUB_NESTED_ALONG_WITH_SIBLINGS_TLV,
                   expected_op_print=test_tlv_data.STR_OP_MULTI_BYTE_TAG_FAKE_SUB_NESTED_ALONG_WITH_SIBLINGS_TLV,
                   test_name='Multi Byte Tag, Fake Sub Nested, along with Siblings TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_FAKE_MULTIBYTE_TAG_IN_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_FAKE_MULTIBYTE_TAG_IN_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_FAKE_MULTIBYTE_TAG_IN_NESTED_TLV,
                   test_name='Fake MultiByte Tag in Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_FAKE_MULTIBYTE_LENGTH_IN_NESTED_TLV,
                   expected_op=test_tlv_data.LIST_OP_FAKE_MULTIBYTE_LENGTH_IN_NESTED_TLV,
                   expected_op_print=test_tlv_data.STR_OP_FAKE_MULTIBYTE_LENGTH_IN_NESTED_TLV,
                   test_name='Fake MultiByte Length in Nested TLV',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_NESTED_TLVS_WITH_ASCII_DATA,
                   expected_op=test_tlv_data.LIST_OP_NESTED_TLVS_WITH_ASCII_DATA,
                   expected_op_print=test_tlv_data.STR_OP_NESTED_TLVS_WITH_ASCII_DATA,
                   test_name='Nested TLVs with ascii data; value_in_ascii=True',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_NESTED_TLVS_WITH_ASCII_DATA,
                   expected_op=test_tlv_data.LIST_OP_NESTED_TLVS_WITH_ASCII_DATA,
                   expected_op_print=test_tlv_data.STR_OP_NESTED_TLVS_WITH_ASCII_DATA_VALUE_IN_ASCII_IS_FALSE,
                   test_name='Nested TLVs with ascii data; value_in_ascii=False',
                   length_in_decimal=True,
                   value_in_ascii=False,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_NESTED_TLV_MULTI_BYTE_LEN_81,
                   expected_op=test_tlv_data.LIST_OP_NESTED_TLV_MULTI_BYTE_LEN_81,
                   expected_op_print=test_tlv_data.STR_OP_NESTED_TLV_MULTI_BYTE_LEN_81,
                   test_name='Nested TLV, Multi Byte Len 81; length_in_decimal=True',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_NESTED_TLV_MULTI_BYTE_LEN_81,
                   expected_op=test_tlv_data.LIST_OP_NESTED_TLV_MULTI_BYTE_LEN_81,
                   expected_op_print=test_tlv_data.STR_OP_NESTED_TLV_MULTI_BYTE_LEN_81_LENGTH_IN_DECIMAL_IS_FALSE,
                   test_name='Nested TLV, Multi Byte Len 81; length_in_decimal=False',
                   length_in_decimal=False,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_NESTED_TLV_MULTI_BYTE_LEN_82,
                   expected_op=test_tlv_data.LIST_OP_NESTED_TLV_MULTI_BYTE_LEN_82,
                   expected_op_print=test_tlv_data.STR_OP_NESTED_TLV_MULTI_BYTE_LEN_82,
                   test_name='Nested TLV, Multi Byte Len 82',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NON_NESTED_TLV_DATA_FF,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NON_NESTED_TLV_DATA_FF,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NON_NESTED_TLV_DATA_FF,
                   test_name='Single Non-Nested TLV, Data is FF',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_SINGLE_NON_NESTED_TLV_DATA_FF_BULK,
                   expected_op=test_tlv_data.LIST_OP_SINGLE_NON_NESTED_TLV_DATA_FF_BULK,
                   expected_op_print=test_tlv_data.STR_OP_SINGLE_NON_NESTED_TLV_DATA_FF_BULK,
                   test_name='Single Non-Nested TLV, Data is FF Bulk',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_EMV_TLVS,
                   expected_op=test_tlv_data.LIST_OP_EMV_TLVS,
                   expected_op_print=test_tlv_data.STR_OP_EMV_TLVS,
                   test_name='EMV TLVs',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_EMV_TLVS,
                   expected_op=test_tlv_data.LIST_OP_EMV_TLVS,
                   expected_op_print=test_tlv_data.STR_OP_EMV_TLVS_ONE_LINER_PLAIN,
                   test_name='EMV TLVs One Liner; one_liner=True',
                   length_in_decimal=False,
                   value_in_ascii=False,
                   one_liner=True,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_EMV_TLVS,
                   expected_op=test_tlv_data.LIST_OP_EMV_TLVS,
                   expected_op_print=test_tlv_data.STR_OP_EMV_TLVS_ONE_LINER_IS_FALSE,
                   test_name='EMV TLVs One Liner; one_liner=False',
                   length_in_decimal=False,
                   value_in_ascii=False,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_RSP_TLVS,
                   expected_op=test_tlv_data.LIST_OP_RSP_TLVS,
                   expected_op_print=test_tlv_data.STR_OP_RSP_TLVS,
                   test_name='RSP TLVs',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_RSP_TLVS,
                   expected_op=test_tlv_data.LIST_OP_RSP_TLVS,
                   expected_op_print=test_tlv_data.STR_OP_RSP_TLVS_ONE_LINER_PLAIN,
                   test_name='RSP TLVs One Liner',
                   length_in_decimal=False,
                   value_in_ascii=False,
                   one_liner=True,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_AID_00_TAG,
                   expected_op=test_tlv_data.LIST_OP_AID_00_TAG_NON_TLV_NEIGHBOUR,
                   expected_op_print=test_tlv_data.STR_OP_AID_00_TAG_NON_TLV_NEIGHBOUR,
                   test_name='AID, 00 Tag, non_tlv_neighbor=True',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_AID_00_TAG,
                   expected_op=test_tlv_data.LIST_OP_AID_00_TAG,
                   expected_op_print=test_tlv_data.STR_OP_AID_00_TAG,
                   test_name='AID, 00 Tag, non_tlv_neighbor=False',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=False,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_AID_01_TAG,
                   expected_op=test_tlv_data.LIST_OP_AID_01_TAG,
                   expected_op_print=test_tlv_data.STR_OP_AID_01_TAG,
                   test_name='AID, 01 Tag',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_RESTRICT_CHILD_MAPPING,
                   expected_op=test_tlv_data.LIST_OP_RESTRICT_CHILD_MAPPING,
                   expected_op_print=test_tlv_data.STR_OP_RESTRICT_CHILD_MAPPING,
                   test_name='Restrict Child Mapping',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_RESTRICT_TWO_BYTE_TAG,
                   expected_op=test_tlv_data.LIST_OP_RESTRICT_TWO_BYTE_TAG,
                   expected_op_print=test_tlv_data.STR_OP_RESTRICT_TWO_BYTE_TAG,
                   test_name='Restrict Two Byte Tag',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_PROFILE_1,
                   expected_op=test_tlv_data.LIST_OP_PROFILE_1,
                   expected_op_print=test_tlv_data.STR_OP_PROFILE_1,
                   test_name='Profile 1',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_LEN_ADDITIONAL_BYTES_AS_0,
                   expected_op=test_tlv_data.LIST_OP_LEN_ADDITIONAL_BYTES_AS_0,
                   expected_op_print=test_tlv_data.STR_OP_LEN_ADDITIONAL_BYTES_AS_0,
                   test_name='Length as F0, Len Additional bytes are 0',
                   length_in_decimal=True,
                   value_in_ascii=True,
                   one_liner=False,
                   non_tlv_neighbor=True,
                   ),

        TestObjTlv(input_data=test_tlv_data.STR_INP_BASE_64_TLV,
                   expected_op=test_tlv_data.LIST_OP_BASE_64_TLV,
                   expected_op_print=test_tlv_data.STR_OP_BASE_64_TLV,
                   test_name='Base 64',
                   ),
    ]

    @unittest.mock.patch('sys.stdout', new_callable=io.StringIO)
    def assert_stdout(self, test_obj, mock_stdout):
        expected_op_print = test_obj.expected_op_print[1:] if test_obj.expected_op_print[
                                                                  0] == '\n' else test_obj.expected_op_print
        tlv_handler_result = TlvHandler(input_data=test_obj.input_data,
                                        non_tlv_neighbor=test_obj.non_tlv_neighbor).process_tlv()
        TlvParser(tlv_handler_result=tlv_handler_result).get_printable_tlv(
            length_in_decimal=test_obj.length_in_decimal,
            value_in_ascii=test_obj.value_in_ascii,
            one_liner=test_obj.one_liner,
            print_also=True)
        self.assertEqual(mock_stdout.getvalue(), expected_op_print)

    def test_tlv_data(self):
        """
        :return:
        """
        self.maxDiff = None
        for count, test_obj in enumerate(TestTlv.test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + (test_obj.test_name if test_obj.test_name else str(count))):
                tlv_handler_result = TlvHandler(input_data=test_obj.input_data,
                                                non_tlv_neighbor=test_obj.non_tlv_neighbor).process_tlv()
                tlv_obj = tlv_handler_result.get(PhKeys.RESULT_PROCESSED)
                self.assertEqual(tlv_obj, test_obj.expected_op)

    def test_tlv_print(self):
        """

        :return:
        """
        self.maxDiff = None
        for count, test_obj in enumerate(TestTlv.test_obj_pool, start=1):
            with self.subTest(STR_TEST_OBJ + (test_obj.test_name if test_obj.test_name else str(count))):
                self.assert_stdout(test_obj)


if __name__ == '__main__':
    unittest.main()



import os

from play_helpers.ph_util import PhUtil

from tlv_play import PACKAGE_NAME


class Folders:
    top_folder_path = [os.pardir, os.pardir]

    DIR_DATA = 1

    DIR_USER_DATA = 2
    DIR_USER_DATA_GENERIC = 24

    DIR_SAMPLE_DATA = 3
    DIR_SAMPLE_DATA_GENERIC = 34

    DIR_RES = 8
    DIR_RES_IMAGES = 81

    DIR_TEST = 9
    DIR_TEST_LOGS = 91

    LOCATIONS_MAPPING = {
        #
        DIR_RES: [PACKAGE_NAME, 'res'],
        DIR_RES_IMAGES: [PACKAGE_NAME, 'res', 'images'],
        #
        DIR_DATA: ['data'],
        #
        DIR_SAMPLE_DATA: ['data', 'sample_data'],
        DIR_SAMPLE_DATA_GENERIC: ['data', 'sample_data', 'Generic'],
        #
        DIR_USER_DATA: ['data', 'user_data'],
        DIR_USER_DATA_GENERIC: ['data', 'user_data', 'Generic'],
        #
        DIR_TEST: [PACKAGE_NAME, 'test'],
        DIR_TEST_LOGS: [PACKAGE_NAME, 'test', 'logs'],
    }

    @classmethod
    def in_res_images(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_RES_IMAGES, relative_path)

    @classmethod
    def in_test(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_TEST, relative_path)

    @classmethod
    def in_test_logs(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_TEST_LOGS, relative_path)

    @classmethod
    def in_sample(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_SAMPLE_DATA, relative_path)

    @classmethod
    def in_sample_gen(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_SAMPLE_DATA_GENERIC, relative_path)

    @classmethod
    def in_user(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_USER_DATA, relative_path)

    @classmethod
    def in_user_gen(cls, relative_path=''):
        return cls.__get_path(Folders.DIR_USER_DATA_GENERIC, relative_path)

    @classmethod
    def __get_path(cls, folder_name, relative_path):
        """

        :param folder_name:
        :param relative_path: str or list
        :return:
        """
        if folder_name not in cls.LOCATIONS_MAPPING:
            raise ValueError('Unknown Folder name')
        return os.sep.join(
            filter(None, PhUtil.normalise_list(
                [Folders.top_folder_path, cls.LOCATIONS_MAPPING.get(folder_name), relative_path])))



from tlv_play.main.data_type.data_type_master import DataTypeMaster
from tlv_play.main.helper.data import Data


class UnitTesting(DataTypeMaster):

    def __init__(self):
        super().__init__()

    def set_print_input(self):
        print_input = None
        super().set_print_input(print_input)

    def set_print_output(self):
        print_output = None
        super().set_print_output(print_output)

    def set_print_info(self):
        print_info = None
        super().set_print_info(print_info)

    def set_quiet_mode(self):
        quite_mode = None
        super().set_quiet_mode(quite_mode)

    def set_remarks(self):
        remarks = None
        super().set_remarks(remarks)

    def set_encoding(self):
        encoding = None
        super().set_encoding(encoding)

    def set_encoding_errors(self):
        encoding_errors = None
        super().set_encoding_errors(encoding_errors)

    def set_output_path(self):
        output_path = None
        super().set_output_path(output_path)

    def set_output_file_name_keyword(self):
        output_file_name_keyword = None
        super().set_output_file_name_keyword(output_file_name_keyword)

    def set_archive_output(self):
        archive_output = None
        super().set_archive_output(archive_output)

    def set_archive_output_format(self):
        archive_output_format = None
        super().set_archive_output_format(archive_output_format)

    def set_length_in_decimal(self):
        length_in_decimal = None
        super().set_length_in_decimal(length_in_decimal)

    def set_value_in_ascii(self):
        value_in_ascii = None
        super().set_value_in_ascii(value_in_ascii)

    def set_one_liner(self):
        one_liner = None
        super().set_one_liner(one_liner)

    def set_non_tlv_neighbor(self):
        non_tlv_neighbor = None
        super().set_non_tlv_neighbor(non_tlv_neighbor)

    def set_data_pool(self):
        data_pool_positive = [
            #
            Data(
                remarks='BER TLV; Multiple Simple TLV as Children',
                input_data='D00D8103010500820281829902090A'
            ),
            #
            Data(
                remarks='Simple TLV; length_in_decimal=True',
                input_data='86020102',
                length_in_decimal=True,
            ),
            #
            Data(
                remarks='Simple TLV; length_in_decimal=False',
                input_data='86020102',
                length_in_decimal=False,
            ),
            #
            Data(
                remarks='BER TLV; Multiple Simple TLV as Children; one_liner=True',
                input_data='D00D 8103010500 82028182 9902090A',
                one_liner=True
            ),
            #
            Data(
                remarks='BER TLV; Multiple Simple TLV as Children; one_liner=False',
                input_data='D00D 8103010500 82028182 9902090A',
                one_liner=False
            ),
            #
            Data(
                remarks='Simple TLV with ASCII Characters; value_in_ascii=True',
                input_data='50 04 5553494D',
                value_in_ascii=True,
            ),
            #
            Data(
                remarks='Simple TLV with ASCII Characters; value_in_ascii=False',
                input_data='50 04 5553494D',
                value_in_ascii=False,
            ),
            #
            Data(
                remarks='List',
                input_data=['50 04 5553494D', 'D00D 8103010500 82028182 9902090A', '86020102',
                            'D00D8103010500820281829902090A'],
            ),
            #
            Data(
                remarks='Test TLV',
                input_data="""
            9F4005FF80F0F001
        91102263BCC1C2D9C4420013
        9F0206000000012345
        9F0306000000004000
        9F26088E19ED4BCA5C670A
        82025C00
        5F340102
        9F3602000A
        9F0702FF00
        9F080208C1
        9F09021001
        8A025931
        9F3403A40002
        9F270180
        9F1E0853455249414C3132
        9F0D05F040008800
        9F0E05FCF8FCF8F0
        9F0F05FCF8FCF8F0
        5F28020840
        9F390100
        9F1A020840
        9F350122
        95050000048000
        5F2A020840
        9B024800
        9F2103123456
        9C0100
        9F370400BC614E
        4F07A0000000031010
        9F0607A0000000031010
        9F7C0412345678
        8407A0000000031010
        9F1006010A03600000
        9F5B052000000000
        9F4104000001B3
        910A2263BCC1C2D9C4420013
        710A0102030405060708090A
        720A0102030405060708090A
            """,
                one_liner=False,
            ),
            #
            Data(
                remarks='Nested BER TLV; length_in_decimal=True; value_in_ascii=True; one_liner=True',
                input_data='064B21220D2048656C6C6F2C204275792031204742204461746120666F7220302E3520555344210F0D0D41726520596F7520537572653F151431107777772E66616365626F6F6B2E636F6D0500',
                length_in_decimal=True,
                value_in_ascii=True,
                one_liner=True,
            ),
            #
            Data(
                remarks='Multiple Simple TLVs; Neighbors; Non TLV Neighbors',
                input_data='810201020102',
            ),
            #
            Data(
                remarks='Multiple Simple TLVs; Neighbors; Non TLV Neighbors',
                input_data='81020102810201020102',
            ),
            #
            Data(
                remarks='Non TLV',
                input_data='0102',
            ),
            #
        ]
        data_pool_negative = [
            #
            Data(
                remarks_list='Kwargs; remarks_list',
                input_data='86020102',
            ),
            #
            Data(
                remarks='Kwargs; raw_data',
                raw_data='86020102',
            ),
            #
            Data(
                remarks='Kwargs; future',
                input_data='86020102',
                future='AmenityPj.in',
            ),
            #
            Data(
                remarks='Empty Data',
                input_data='',
            ),
            #
            Data(
                remarks='Garbage Hex Data',
                input_data='124',
            ),
        ]
        #
        super().set_data_pool(
            data_pool_positive
            + data_pool_negative
        )



from play_helpers.ph_defaults import PhDefaults, PhDefaultTypesInclude, PhDefaultTypesExclude
from play_helpers.ph_file_extensions import PhFileExtensions


class Defaults:
    #############
    # Generic Objects
    #############
    EXECUTION_MODE = PhDefaults.EXECUTION_MODE
    ERROR_HANDLING_MODE = PhDefaults.ERROR_HANDLING_MODE
    OUTPUT_FILE_EXT = PhFileExtensions.TXT
    #############
    # Data Objects
    #############
    # Common Objects
    # INPUT_DATA
    PRINT_INPUT = PhDefaults.PRINT_INPUT
    PRINT_OUTPUT = PhDefaults.PRINT_OUTPUT
    PRINT_INFO = PhDefaults.PRINT_INFO
    QUITE_MODE = PhDefaults.QUITE_MODE
    # REMARKS
    ENCODING = PhDefaults.CHAR_ENCODING
    ENCODING_ERRORS = PhDefaults.ENCODING_ERRORS
    OUTPUT_PATH = PhDefaults.OUTPUT_PATH
    OUTPUT_FILE_NAME_KEYWORD = PhDefaults.OUTPUT_FILE_NAME_KEYWORD
    ARCHIVE_OUTPUT = PhDefaults.ARCHIVE_OUTPUT
    ARCHIVE_OUTPUT_FORMAT = PhDefaults.ARCHIVE_OUTPUT_FORMAT
    # Specific Objects
    LENGTH_IN_DECIMAL = False
    VALUE_IN_ASCII = False
    ONE_LINER = True
    NON_TLV_NEIGHBOR = True


class DefaultTypesInclude:
    # Common Objects
    # INPUT_DATA
    PRINT_INPUT = PhDefaultTypesInclude.PRINT_INPUT
    PRINT_OUTPUT = PhDefaultTypesInclude.PRINT_OUTPUT
    PRINT_INFO = PhDefaultTypesInclude.PRINT_INFO
    QUITE_MODE = PhDefaultTypesInclude.QUITE_MODE
    # REMARKS
    ENCODING = PhDefaultTypesInclude.ENCODING
    ENCODING_ERRORS = PhDefaultTypesInclude.ENCODING_ERRORS
    OUTPUT_PATH = PhDefaultTypesInclude.OUTPUT_PATH
    ARCHIVE_OUTPUT = PhDefaultTypesInclude.ARCHIVE_OUTPUT
    ARCHIVE_OUTPUT_FORMAT = PhDefaultTypesInclude.ARCHIVE_OUTPUT_FORMAT
    # Specific Objects
    LENGTH_IN_DECIMAL = bool
    VALUE_IN_ASCII = bool
    ONE_LINER = bool
    NON_TLV_NEIGHBOR = bool


class DefaultTypesExclude:
    # Common Objects
    INPUT_DATA: PhDefaultTypesExclude.INPUT_DATA



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_data_master import PhMasterData, PhMasterDataKeys
from play_helpers.ph_defaults import PhDefaultTypesExclude
from play_helpers.ph_exception_helper import PhExceptionHelper
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil

from tlv_play.main.helper.data import Data
from tlv_play.main.helper.defaults import Defaults, DefaultTypesInclude
from tlv_play.main.helper.keywords import KeyWords


def print_data(data=None, meta_data=None, info_data=None, master_data=None):
    """
    
    :param data:
    :param meta_data:
    :param info_data:
    :param master_data:
    :return:
    """
    if master_data is not None and isinstance(master_data, PhMasterData):
        data = master_data.get_master_data(PhMasterDataKeys.DATA)
        meta_data = master_data.get_master_data(PhMasterDataKeys.META_DATA)
        info_data = master_data.get_master_data(PhMasterDataKeys.INFO_DATA)
    if data.quite_mode:
        return
    input_sep = PhConstants.SEPERATOR_ONE_LINE
    output_sep = PhConstants.SEPERATOR_MULTI_LINE
    len_sep = PhConstants.SEPERATOR_ONE_LINE
    if data.print_info:
        remarks_original = data.get_remarks_as_str(user_original_remarks=True)
        remarks_generated = data.get_remarks_as_str()
        remarks_generated_stripping_needed = True if remarks_generated.endswith(
            PhConstants.DEFAULT_TRIM_STRING) else False
        if remarks_original:
            if remarks_generated_stripping_needed:
                if remarks_generated.strip(PhConstants.DEFAULT_TRIM_STRING) in remarks_original:
                    remarks_generated = ''
            else:
                if remarks_original in remarks_generated:
                    remarks_generated = ''
            meta_data.output_dic.update(
                PhUtil.get_dic_data_and_print(PhKeys.REMARKS, PhConstants.SEPERATOR_ONE_LINE, remarks_original))
        if remarks_generated:
            meta_data.output_dic.update(
                PhUtil.get_dic_data_and_print(PhKeys.REMARKS_GENERATED, PhConstants.SEPERATOR_ONE_LINE,
                                              remarks_generated))
        if info_data is not None:
            info_count = info_data.get_info_count()
            info_msg = info_data.get_info_str()
            info_present = info_msg
            if info_present:
                sep = PhConstants.SEPERATOR_MULTI_LINE_TABBED if info_count > 1 else PhConstants.SEPERATOR_ONE_LINE
                meta_data.output_dic.update(PhUtil.get_dic_data_and_print(PhKeys.INFO_DATA, sep, info_msg))
                if data.print_info:
                    meta_data.output_dic.update(
                        PhUtil.get_dic_data_and_print(PhKeys.INFO_DATA, len_sep, info_msg, length_needed=True))
        info_contents_pool = {
            PhKeys.TRANSACTION_ID: meta_data.transaction_id,
            PhKeys.LENGTH_IN_DECIMAL: data.length_in_decimal,
            PhKeys.VALUE_IN_ASCII: data.value_in_ascii,
            PhKeys.ONE_LINER: data.one_liner,
            PhKeys.NON_TLV_NEIGHBOR: data.non_tlv_neighbor,
            PhKeys.OUTPUT_PATH: data.output_path,
            PhKeys.OUTPUT_FILE_NAME_KEYWORD: data.output_file_name_keyword,
            PhKeys.ENCODING: data.encoding,
            PhKeys.ENCODING_ERRORS: data.encoding_errors,
            PhKeys.ARCHIVE_OUTPUT: data.archive_output,
            PhKeys.ARCHIVE_OUTPUT_FORMAT: data.archive_output_format,
            PhKeys.QUITE_MODE: data.quite_mode,
        }
        meta_data.output_dic.update(PhUtil.get_dic_data_and_print(PhKeys.INFO, PhConstants.SEPERATOR_INFO,
                                                                  PhUtil.decorate_info_data(info_contents_pool)))
    in_data = meta_data.input_data_org
    if data.print_input:
        meta_data.output_dic.update(PhUtil.get_dic_data_and_print(PhKeys.INPUT_DATA, input_sep, in_data))
    if data.print_info:
        meta_data.output_dic.update(
            PhUtil.get_dic_data_and_print(PhKeys.INPUT_DATA, len_sep, in_data, length_needed=True))
    output_present = meta_data.parsed_data
    if output_present:
        if data.print_output:
            meta_data.output_dic.update(
                PhUtil.get_dic_data_and_print(PhKeys.OUTPUT_DATA, output_sep, meta_data.parsed_data))
        if data.print_info:
            meta_data.output_dic.update(
                PhUtil.get_dic_data_and_print(PhKeys.OUTPUT_DATA, len_sep, meta_data.parsed_data, length_needed=True))
    PhUtil.print_separator()


def set_includes_excludes_files(data, meta_data):
    """

    :param data:
    :param meta_data:
    """
    # Always exclude output files
    meta_data.excludes = ['*_' + KeyWords.OUTPUT_FILE_NAME_KEYWORD + '.*']
    # Include Everything for now


def dict_to_data(config_data):
    data_types_include = {
        # Common Param
        # PhKeys.INPUT_DATA:
        PhKeys.PRINT_INPUT: DefaultTypesInclude.PRINT_INPUT,
        PhKeys.PRINT_OUTPUT: DefaultTypesInclude.PRINT_OUTPUT,
        PhKeys.PRINT_INFO: DefaultTypesInclude.PRINT_INFO,
        PhKeys.QUITE_MODE: DefaultTypesInclude.QUITE_MODE,
        PhKeys.ENCODING: DefaultTypesInclude.ENCODING,
        PhKeys.ENCODING_ERRORS: DefaultTypesInclude.ENCODING_ERRORS,
        PhKeys.ARCHIVE_OUTPUT: DefaultTypesInclude.ARCHIVE_OUTPUT,
        PhKeys.ARCHIVE_OUTPUT_FORMAT: DefaultTypesInclude.ARCHIVE_OUTPUT_FORMAT,
        # Specific Param
        PhKeys.LENGTH_IN_DECIMAL: DefaultTypesInclude.LENGTH_IN_DECIMAL,
        PhKeys.VALUE_IN_ASCII: DefaultTypesInclude.VALUE_IN_ASCII,
        PhKeys.ONE_LINER: DefaultTypesInclude.ONE_LINER,
        PhKeys.NON_TLV_NEIGHBOR: DefaultTypesInclude.NON_TLV_NEIGHBOR,
    }
    data_types_exclude = {
        # Common Param
        PhKeys.INPUT_DATA: PhDefaultTypesExclude.INPUT_DATA,
        # PhKeys.REMARKS: ,
    }

    config_data = PhUtil.dict_to_data(user_dict=config_data, data_types_include=data_types_include,
                                      data_types_exclude=data_types_exclude, trim_data=False)
    # PhUtil.print_iter(config_data, 'config_data initial', verbose=True)
    for k, v in config_data.items():
        if not v:
            continue
        config_data[k] = v
    # PhUtil.print_iter(config_data, 'config_data before cleaning', verbose=True, depth_level=1)
    # PhUtil.print_iter(config_data, 'config_data processed', verbose=True, depth_level=1)
    return config_data


def set_defaults_for_common_objects(data):
    """

    :param data:
    :return:
    """
    # input_data
    data.print_input = PhUtil.set_if_none(data.print_input, Defaults.PRINT_INPUT)
    data.print_output = PhUtil.set_if_none(data.print_output, Defaults.PRINT_OUTPUT)
    data.print_info = PhUtil.set_if_none(data.print_info, Defaults.PRINT_INFO)
    data.quite_mode = PhUtil.set_if_none(data.quite_mode, Defaults.QUITE_MODE)
    # Remarks
    data.encoding = PhUtil.set_if_none(data.encoding, Defaults.ENCODING)
    data.encoding_errors = PhUtil.set_if_none(data.encoding_errors, Defaults.ENCODING_ERRORS)
    data.output_path = PhUtil.set_if_none(data.output_path, Defaults.OUTPUT_PATH)
    data.output_file_name_keyword = PhUtil.set_if_none(data.output_file_name_keyword, Defaults.OUTPUT_FILE_NAME_KEYWORD)
    data.archive_output = PhUtil.set_if_none(data.archive_output, Defaults.ARCHIVE_OUTPUT)
    data.archive_output_format = PhUtil.set_if_none(data.archive_output_format, Defaults.ARCHIVE_OUTPUT_FORMAT)


def set_defaults(data, meta_data):
    """
    Set Default Values if nothing is set
    :param data:
    :param meta_data:
    :return:
    """
    data.length_in_decimal = PhUtil.set_if_none(data.length_in_decimal, Defaults.LENGTH_IN_DECIMAL)
    data.value_in_ascii = PhUtil.set_if_none(data.value_in_ascii, Defaults.VALUE_IN_ASCII)
    data.one_liner = PhUtil.set_if_none(data.one_liner, Defaults.ONE_LINER)
    data.non_tlv_neighbor = PhUtil.set_if_none(data.non_tlv_neighbor, Defaults.NON_TLV_NEIGHBOR)
    if meta_data is None:
        return


def handle_web_request(request_form):
    return Data(**dict_to_data(request_form))


def read_input_file(data, meta_data, info_data):
    try:
        # Text File
        with open(data.input_data, mode='r', encoding=data.encoding, errors=data.encoding_errors) as the_file:
            resp = ''.join(the_file.readlines())
    except UnicodeDecodeError:
        # Binary File/Encoding Error
        with open(data.input_data, 'rb') as the_file:
            resp = the_file.read()
    if not resp:
        raise ValueError(PhExceptionHelper(msg_key=PhConstants.EMPTY_INPUT_FILE))
    return resp


def write_output_file(data, meta_data, info_data, flip_output=False):
    output_file_path = meta_data.output_file_path
    data_to_write = meta_data.parsed_data
    if flip_output is True:
        output_file_path = meta_data.re_output_file_path
        data_to_write = meta_data.re_parsed_data
    data_to_write = PhUtil.set_if_none(data_to_write, PhConstants.STR_EMPTY)
    with open(output_file_path, mode='w', encoding=data.encoding, errors=data.encoding_errors) as file:
        file.writelines(data_to_write)



from play_helpers.ph_util import PhUtil


class Test:

    @classmethod
    def test_sample_data(cls):
        PhUtil.print_separator()

    @classmethod
    def test_all_versions(cls):
        PhUtil.print_separator()

    @classmethod
    def test_all(cls):
        cls.test_sample_data()
        cls.test_all_versions()



class Tlv:
    def __init__(self, tag_list=None, len_list=None, value_list=None, len_dec=0, level=0):
        if tag_list is None:
            tag_list = []
        if len_list is None:
            len_list = []
        if value_list is None:
            value_list = []
        self.tag_list = tag_list
        self.len_list = len_list
        self.value_list = value_list
        self.len_dec = len_dec
        self.level = level

    def __repr__(self):
        return f'TLV(tag_list={self.tag_list}, len_list={self.len_list}, value_list={self.value_list}, len_dec= {self.len_dec},level= {self.level}'

    def __eq__(self, other):
        result = False
        if isinstance(other, Tlv):
            result = self.tag_list == other.tag_list \
                     and self.len_list == other.len_list \
                     and self.len_dec == other.len_dec \
                     and self.level == other.level
            if result:
                # TODO: Nested Objects / list is not supported
                result = self.value_list == other.value_list
        return result



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_util import PhUtil


class TlvPrint:
    SPACE = '  '
    SPACE_ONE_LINER = ' '
    SPACE_TL = ' '
    SEP_TL_V = '\n'
    SEP_TL_V_ONE_LINER = ''
    SEP_CONCAT_TLVS = '\n'
    SEP_SUB_TLVS = '\n'

    def __init__(self, tlv_obj, level, length_in_decimal=None, value_in_ascii=None, one_liner=None):
        self.tlv_obj = tlv_obj
        self.level = level
        self.length_in_decimal = length_in_decimal
        self.value_in_ascii = value_in_ascii
        self.one_liner = one_liner

    def get_tl_as_str(self):
        return (f'{TlvPrint.SPACE * self.level}'
                f'{PhUtil.to_hex_string(self.tlv_obj.tag_list, PhConstants.FORMAT_HEX_STRING_AS_PACK)}'
                f'{TlvPrint.SPACE_TL}'
                f'{PhUtil.to_hex_string(self.tlv_obj.len_list, PhConstants.FORMAT_HEX_STRING_AS_PACK)}') \
            + (f' ({self.tlv_obj.len_dec})' if self.length_in_decimal else '')

    def get_v_as_str(self):
        value_str = PhUtil.to_hex_string(self.tlv_obj.value_list, PhConstants.FORMAT_HEX_STRING_AS_PACK)
        value_str_ascii = PhUtil.hex_str_to_ascii(value_str) if value_str and self.value_in_ascii else ''
        space = TlvPrint.SPACE_ONE_LINER if self.one_liner else TlvPrint.SPACE * (self.level + 1)
        return (f'{space}{value_str}' if value_str else '') + (f' ({value_str_ascii})' if value_str_ascii else '')

    def get_tlv_as_str(self):
        sep = TlvPrint.SEP_TL_V_ONE_LINER if self.one_liner else TlvPrint.SEP_TL_V
        return sep.join(filter(None, [f'{self.get_tl_as_str()}', f'{self.get_v_as_str()}']))



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil

from tlv_play.main.tlv.tlv import Tlv
from tlv_play.main.tlv.tlv_print import TlvPrint


class TlvParser:
    def __init__(self, tlv_handler_result):
        self.tlv_obj = tlv_handler_result.get(PhKeys.RESULT_PROCESSED)
        self.non_tlv_obj = tlv_handler_result.get(PhKeys.RESULT_UNPROCESSED, None)

    def get_printable_tlv(self, length_in_decimal=None, value_in_ascii=None, one_liner=None, print_also=False):
        result = self.__parse_tlv(length_in_decimal=length_in_decimal, value_in_ascii=value_in_ascii,
                                  one_liner=one_liner)
        if self.non_tlv_obj is not None:
            result = (f'{result}'
                      f'{PhConstants.SEPERATOR_TWO_LINES_MULTI}'
                      f'Non TLV Neighbor'
                      f'{PhConstants.SEPERATOR_KEY_VALUE}'
                      f'{PhUtil.to_hex_string(self.non_tlv_obj, PhConstants.FORMAT_HEX_STRING_AS_PACK)}')
        if print_also:
            print(result)
        return result

    def __parse_tlv(self, tlv_obj=None, level=0, length_in_decimal=None, value_in_ascii=None, one_liner=None):
        if tlv_obj is None:
            tlv_obj = self.tlv_obj
        parsed_data = ''
        if isinstance(tlv_obj, list):  # List of Multiple Objects
            if isinstance(tlv_obj[0], Tlv):
                for tlv in tlv_obj:
                    parsed_data_temp = self.__parse_tlv_individual(tlv, level=level,
                                                                   length_in_decimal=length_in_decimal,
                                                                   value_in_ascii=value_in_ascii,
                                                                   one_liner=one_liner)
                    parsed_data = TlvPrint.SEP_CONCAT_TLVS.join(filter(None, [parsed_data, parsed_data_temp]))
                return parsed_data
            if isinstance(tlv_obj[0], int):
                return PhUtil.to_hex_string(tlv_obj, PhConstants.FORMAT_HEX_STRING_AS_PACK)
        else:  # Single Object
            return self.__parse_tlv_individual(tlv_obj, level=level, length_in_decimal=length_in_decimal,
                                               value_in_ascii=value_in_ascii, one_liner=one_liner)

    def __parse_tlv_individual(self, tlv, level, length_in_decimal, value_in_ascii, one_liner):
        if isinstance(tlv, list):
            return self.__parse_tlv(tlv, level=level + 1, length_in_decimal=length_in_decimal,
                                    value_in_ascii=value_in_ascii, one_liner=one_liner)
        if not isinstance(tlv, Tlv):
            return ''
        tlv_print = TlvPrint(tlv, level, length_in_decimal, value_in_ascii, one_liner)
        if len(tlv.value_list) > 0 and isinstance(tlv.value_list[0], Tlv):  # Single or Multiple SUB TLV
            parsed_data = f'{tlv_print.get_tl_as_str()}'
            for item in tlv.value_list:
                parsed_data = TlvPrint.SEP_SUB_TLVS.join(
                    filter(None, [parsed_data, self.__parse_tlv(item, level=level + 1,
                                                                length_in_decimal=length_in_decimal,
                                                                value_in_ascii=value_in_ascii,
                                                                one_liner=one_liner)]))
            return parsed_data
        else:
            return tlv_print.get_tlv_as_str()



import os

from play_helpers.ph_constants import PhConstants
from play_helpers.ph_dos import PhDos
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_modes_execution import PhExecutionModes
from play_helpers.ph_process import PhProcess
from play_helpers.ph_util import PhUtil

from tlv_play import MODULE_NAME, PACKAGE_NAME
from tlv_play.main.tlvplay import print_configurations
from tlv_play.test.test_data import TestData


class TestAutoBatch:
    white_listed_tcs = None

    PROJECT_PATH = r'D:\Other\Github_Self\tlvPlay'

    get_file_path_mapping_relative = {
        PhKeys.VAR_EXECUTION_MODE: r'tlv_play\main\tlvplay.py',
        PhKeys.VAR_ERROR_HANDLING_MODE: r'tlv_play\main\tlvplay.py',
        PhKeys.VAR_TOP_FOLDER_PATH: r'tlv_play\main\helper\folders.py',
    }

    get_target_str = {
        PhKeys.VAR_EXECUTION_MODE: 'execution_mode = PhExecutionModes.',
        PhKeys.VAR_ERROR_HANDLING_MODE: 'error_handling_mode = PhErrorHandlingModes.',
        PhKeys.VAR_TOP_FOLDER_PATH: 'top_folder_path = ',
    }

    @classmethod
    def _update_variables_in_file(cls, test_case_data):

        def __update_variable_in_file(_target_file, _target_str, _target_value):
            update_available = False
            current_value = None
            if PhUtil.is_empty(_target_file):
                return None
            if PhUtil.is_empty(_target_str):
                return None
            if PhUtil.is_empty(_target_value):
                return None
            with open(_target_file) as f_read:
                file_data = f_read.read().split('\n')
            for index, item in enumerate(file_data):
                if _target_str in item:
                    item_data = item.split(_target_str)
                    if len(item_data) > 1:
                        current_value = item_data[1]
                        if PhUtil.is_not_empty(current_value) and _target_value != current_value:
                            update_available = True
                            item_data[1] = _target_value
                        if update_available:
                            file_data[index] = _target_str.join(item_data)
                        break
            if update_available:
                with open(_target_file, 'w') as f_write:
                    f_write.writelines('\n'.join(file_data))
                return current_value
            return None

        original_data = {}
        target_keys = [
            PhKeys.VAR_EXECUTION_MODE,
            PhKeys.VAR_ERROR_HANDLING_MODE,
            PhKeys.VAR_TOP_FOLDER_PATH,
        ]
        for target_key in target_keys:
            if target_key not in test_case_data:
                continue
            target_str = TestAutoBatch.get_target_str.get(target_key)
            target_value = test_case_data.get(target_key)
            target_file = os.sep.join(
                [TestAutoBatch.PROJECT_PATH, TestAutoBatch.get_file_path_mapping_relative.get(target_key)])
            value_original = __update_variable_in_file(target_file, target_str, target_value)
            if value_original is not None:
                print(f'{target_key}: value_original was {value_original}; target_value is {target_value}')
                original_data.update({target_key: value_original})
        return original_data

    @classmethod
    def clean_up(cls, original_data):
        cls._update_variables_in_file(original_data)

    @classmethod
    def prepare_batch_file(cls, test_case_data, default_batch_data=None, batch_params=None):
        """

        :param default_batch_data:
        :param test_case_data:
        :param batch_params:
        :return:
        """
        log_file_path = os.sep.join([PACKAGE_NAME, 'test', 'logs', test_case_data.get(PhKeys.TEST_CASE_FILE_NAME)])
        executable_script = [
            default_batch_data,
            PhDos.echo_off(),
            PhDos.switch_to_current_folder(),
            PhDos.get_seperator(test_case_data.get(PhKeys.TEST_CASE_ID)),
            PhDos.change_directory_parent(),
            PhDos.change_directory_parent(),
            PhDos.create_directory(file_path=log_file_path),
            PhDos.call_script_for_env_handling(True),
            PhConstants.SEPERATOR_TWO_WORDS.join(filter(None, [
                PhDos.run_python(module_name=MODULE_NAME),
                batch_params,
                PhDos.redirect_output(file_path=log_file_path)
            ])),
            PhDos.call_script_for_env_handling(False),
            PhDos.get_seperator(heading="Batch Execution Done"),
            PhDos.echo_on(),
        ]
        batch_file_path = os.sep.join([PhDos.BATCH_RUN_TC])
        with open(batch_file_path, mode='w') as my_file:
            content = PhUtil.normalise_list(executable_script)
            my_file.write('\n'.join(content))
        PhProcess.run_batch_file(batch_file_path)

    @classmethod
    def tc_is_not_whitelisted(cls, key):
        if TestAutoBatch.white_listed_tcs is None or len(TestAutoBatch.white_listed_tcs) == 0:
            return False
        key_name = PhExecutionModes.get_key_name(key)
        if key_name in TestAutoBatch.white_listed_tcs:
            return False
        return True

    @classmethod
    def test(cls, test_case_data, default_batch_data=None):
        """

        :param test_case_data:
        :return:
        """
        PhUtil.print_heading(test_case_data.get(PhKeys.TEST_CASE_ID))
        PhUtil.print_iter(test_case_data, header='test_case_data')
        PhUtil.print_heading('update_variables_in_file', heading_level=2)
        original_data = cls._update_variables_in_file(test_case_data)
        cls.prepare_batch_file(test_case_data, default_batch_data)
        PhUtil.print_heading('clean_up', heading_level=2)
        cls.clean_up(original_data)

    @classmethod
    def test_cli(cls, test_case_data, default_batch_data=None):
        """

        :param test_case_data:
        :return:
        """
        PhUtil.print_heading(test_case_data.get(PhKeys.TEST_CASE_ID))
        PhUtil.print_iter(test_case_data, header='test_case_data')
        batch_params = test_case_data.get(PhKeys.BATCH_PARAMS)
        cls.prepare_batch_file(test_case_data, default_batch_data, batch_params)

    @classmethod
    def test_all(cls):
        """

        :return:
        """
        print_configurations()
        """
        Non CLI Tests
        """
        for index, key in enumerate(TestData.dynamic_data.keys()):
            if cls.tc_is_not_whitelisted(key):
                continue
            test_case_data = TestData.get_test_data(key=key)
            common_data = PhUtil.to_list(PhDos.echo(f'Iteration {index + 1}', wrap_up=True))
            common_data.extend(PhDos.common_info())
            cls.test(test_case_data=test_case_data, default_batch_data=common_data)
        """
        CLI Tests
        """
        TestData.generate_dynamic_cli_from_read_me()
        for index, key in enumerate(TestData.dynamic_data_cli.keys()):
            if cls.tc_is_not_whitelisted(key):
                continue
            test_case_data = TestData.get_test_data_cli(key=key)
            common_data = PhUtil.to_list(PhDos.echo(f'Iteration {index + 1}', wrap_up=True))
            common_data.extend(PhDos.common_info())
            cls.test_cli(test_case_data=test_case_data, default_batch_data=common_data)


def main():
    """

    :return:
    """
    TestAutoBatch.white_listed_tcs = [
        # PhExecutionModes.ALL,
        # unit_testing',
    ]
    TestAutoBatch.test_all()


if __name__ == '__main__':
    main()



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_modes_execution import PhExecutionModes
from play_helpers.ph_util import PhUtil


class TestData:
    # Unit Testing Sequences
    dynamic_data = {
        PhExecutionModes.UNIT_TESTING:
            {
                PhKeys.VAR_EXECUTION_MODE: 'UNIT_TESTING',
            },
        PhExecutionModes.USER:
            {
                PhKeys.VAR_EXECUTION_MODE: 'USER',
            },
        PhExecutionModes.SAMPLES_LIST:
            {
                PhKeys.VAR_EXECUTION_MODE: 'SAMPLES_LIST',
            },
        PhExecutionModes.DEV:
            {
                PhKeys.VAR_EXECUTION_MODE: 'DEV',
            },
        PhExecutionModes.KNOWN_ISSUES:
            {
                PhKeys.VAR_EXECUTION_MODE: 'KNOWN_ISSUES',
            },
        PhExecutionModes.UNIT_TESTING_EXTERNAL:
            {
                PhKeys.VAR_EXECUTION_MODE: 'UNIT_TESTING_EXTERNAL',
            },
        PhExecutionModes.ALL:
            {
                PhKeys.VAR_EXECUTION_MODE: 'ALL',
            },
    }

    default_data = {
        PhKeys.VAR_EXECUTION_MODE: 'ALL',
        PhKeys.VAR_ERROR_HANDLING_MODE: 'CONTINUE_ON_ERROR',
        PhKeys.VAR_TOP_FOLDER_PATH: '[]',
    }

    #
    dynamic_data_cli = {
        'no_param':
            {
                PhKeys.BATCH_PARAMS: '',
                PhKeys.VAR_EXECUTION_MODE: 'USER',
            },
        '--help':
            {
                PhKeys.BATCH_PARAMS: '--help',
                PhKeys.VAR_EXECUTION_MODE: 'USER',
            },
    }

    read_me_cli_pool = [
    ]

    @classmethod
    def get_test_data(cls, key):
        dynamic_data = cls.dynamic_data.get(key, PhConstants.DICT_EMPTY)
        key_name = PhExecutionModes.get_key_name(key)
        for temp_key in cls.default_data:
            if temp_key not in dynamic_data:
                dynamic_data[temp_key] = cls.default_data[temp_key]
        static_data = {
            PhKeys.TEST_CASE_ID: key_name,
            PhKeys.TEST_CASE_NAME: key_name,
            PhKeys.TEST_CASE_FILE_NAME: f'{key_name}.log'
        }
        return PhUtil.dict_merge(static_data, dynamic_data)

    @classmethod
    def get_test_data_cli(cls, key):
        key_name = 'cli_' + key
        dynamic_data = cls.dynamic_data_cli.get(key, PhConstants.DICT_EMPTY)
        static_data = {
            PhKeys.TEST_CASE_ID: key_name,
            PhKeys.TEST_CASE_NAME: key_name,
            PhKeys.TEST_CASE_FILE_NAME: f'{key_name}.log'
        }
        return PhUtil.dict_merge(static_data, dynamic_data)

    @classmethod
    def generate_dynamic_cli_from_read_me(cls):
        for index, batch_param in enumerate(TestData.read_me_cli_pool):
            TestData.dynamic_data_cli.update({f'read_me_{index}': {PhKeys.BATCH_PARAMS: f'"{batch_param}"'}})



import copy
import os

from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil

from tlv_play.main.convert import converter
from tlv_play.main.convert.handler import handle_data
from tlv_play.main.helper.infodata import InfoData
from tlv_play.main.helper.metadata import MetaData


def process_all_data_types(data, meta_data=None, info_data=None):
    """

    :param info_data:
    :param meta_data:
    :param data:
    :return:
    """
    converter.set_defaults_for_common_objects(data)
    if meta_data is None:
        meta_data = MetaData(input_data_org=data.input_data)
    if info_data is None:
        info_data = InfoData()
    """
    Bulk Data Handling (Recursive; List / Tuple / Dict)
    """
    multiple_inputs = False
    if isinstance(data.input_data, list):
        # List is provided
        multiple_inputs = True
        meta_data.input_mode_key = PhKeys.INPUT_LIST
    if isinstance(data.input_data, tuple):
        # CUI (Click) Multi is a tuple
        multiple_inputs = True
        meta_data.input_mode_key = PhKeys.INPUT_TUPLE
    if isinstance(data.input_data, dict):
        multiple_inputs = True
        dict_format = True
        meta_data.input_mode_key = PhKeys.INPUT_DICT
    if multiple_inputs:
        elements_count = len(data.input_data)
        data.append_input_modes_hierarchy(meta_data.input_mode_key)
        data.set_auto_generated_remarks_if_needed()
        data.set_one_time_remarks(f'({elements_count} Elements)')
        PhUtil.print_heading(data.get_remarks_as_str(), heading_level=3)
        converter.print_data(data, meta_data)
        parsed_data_list = []
        actual_remarks_length = len(data.remarks)
        current_remarks = PhUtil.extend_list(data.remarks, expected_length=len(data.input_data))
        iterator = data.input_data
        for index, input_data_item in enumerate(iterator, start=1):
            sub_data = copy.deepcopy(data)
            sub_data.input_data = input_data_item
            sub_data.reset_auto_generated_remarks()
            sub_data.set_extended_remarks_available(False if index <= actual_remarks_length else True)
            sub_data.set_user_remarks(current_remarks[index - 1])
            if elements_count > 1:
                sub_data.set_auto_generated_remarks_if_needed(
                    PhUtil.get_key_value_pair(key='item', value=index, sep=PhConstants.SEPERATOR_TWO_WORDS,
                                              dic_format=False))
            parsed_data_list.append(process_all_data_types(sub_data))
        meta_data.parsed_data = parsed_data_list
        return meta_data.parsed_data
    """
    Dir Handling (Bulk Mode)
    """
    if data.input_data and isinstance(data.input_data, str) and os.path.isdir(os.path.abspath(data.input_data)):
        # directory is provided
        meta_data.input_mode_key = PhKeys.INPUT_DIR
        data.append_input_modes_hierarchy(meta_data.input_mode_key)
        PhUtil.print_heading(data.get_remarks_as_str(), heading_level=3)
        converter.set_defaults(data, meta_data)
        converter.print_data(data, meta_data)
        converter.set_includes_excludes_files(data, meta_data)
        files_list = PhUtil.traverse_it(top=os.path.abspath(data.input_data), traverse_mode='Regex',
                                        include_files=meta_data.include_files, excludes=meta_data.excludes)
        if files_list:
            files_list_data = data
            files_list_data.input_data = files_list
            return process_all_data_types(files_list_data)
    """
    Individual File Handling
    """
    file_dic_all_str = {}
    data.set_auto_generated_remarks_if_needed()
    PhUtil.print_heading(data.get_remarks_as_str(), heading_level=2)
    if data.input_data and isinstance(data.input_data, str) and os.path.isfile(os.path.abspath(data.input_data)):
        # file is provided
        resp = converter.read_input_file(data=data, meta_data=meta_data, info_data=info_data)
        # Treat all Files Equally; including YML
        meta_data.input_mode_key = PhKeys.INPUT_FILE
        meta_data.input_file_path = data.input_data
        # converter.set_input_output_format(data)
        data.input_data = resp
        data.append_input_modes_hierarchy(meta_data.input_mode_key)
    """
    Individual Data Handling
    """
    # Needed for a scenario when remarks will be fetched from YML
    data.set_auto_generated_remarks_if_needed()
    converter.set_defaults(data, meta_data)
    data.set_user_remarks_expand_variables()
    """
    Data Processing
    """
    handle_data(data=data, meta_data=meta_data, info_data=info_data)
    converter.print_data(data=data, meta_data=meta_data, info_data=info_data)
    if meta_data.output_file_path:
        PhUtil.make_dirs(file_path=meta_data.output_file_path)
        converter.write_output_file(data=data, meta_data=meta_data, info_data=info_data)
    return meta_data.parsed_data



"""
Provides tlv_play name, title & package information.
"""
TOOL_NAME = 'tlvPlay'
TOOL_TITLE = 'TLV Play'
TOOL_PACKAGE_NAME = 'tlv_play.main'
TOOL_TEST_PACKAGE_NAME = 'tlv_play.test'
MODULE_NAME = 'tlv_play.main.tlvplay'


class InfoData:
    def __init__(self, info=[]):
        self.info = []
        self.set_info(info)

    def set_info(self, info):
        if isinstance(info, list):
            self.info.extend(info)
        else:
            self.info.append(info)

    def get_info_list(self):
        return self.info

    def get_info_count(self):
        return len(self.info)

    def get_info_str(self, sep='\n\t'):
        return sep.join(filter(None, self.info))



import binascii
from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil

from tlv_play.main.tlv.tlv import Tlv

# if b5 b4 b3 b2 b1 are set, then see subsequent bytes
tag_mask_first_byte_subsequent_bytes = 0x1F
# if b8 is set, Another byte follows
tag_mask_subsequent_bytes = 0x80
len_mask_additional_bytes = 0x0F
invalid_tags_list = [[0x00]]


class TlvHandler:
    def __init__(self, input_data, non_tlv_neighbor, info_data=None):
        self.input_data_list = []
        self.non_tlv_neighbor = non_tlv_neighbor
        self.info_data = info_data
        if isinstance(input_data, str):
            input_data = PhUtil.trim_and_kill_all_white_spaces(input_data)
            input_data = PhUtil.decode_to_hex_if_base64(input_data)
            input_data = binascii.unhexlify(input_data)
            self.input_data_list = list(input_data)

    def __process_tlv(self, data_list, level, initial_offset):
        """

        :return:
        """
        offset = initial_offset
        """
        TAG: coded on one or two bytes
        """
        data_len = len(data_list)
        if data_len < 2:
            return None, -1  # Minimum Tag Len is needed
        # Debug
        # print(
        #     f'level: {level}; initial_offset: {initial_offset}; data_list: {PhUtil.to_hex_string(data_list, PhConstants.FORMAT_HEX_STRING_AS_PACK)}')
        tag_list = [data_list[offset]]
        if tag_list[0] & tag_mask_first_byte_subsequent_bytes == tag_mask_first_byte_subsequent_bytes:
            offset += 1
            tag_list.append(data_list[offset])
            while data_list[offset] & tag_mask_subsequent_bytes == tag_mask_subsequent_bytes:
                offset += 1
                if offset >= data_len:
                    return None, -1  # Tag is not available
                tag_list.append(data_list[offset])
        # check for invalid Tags
        for invalid_tag in invalid_tags_list:
            if invalid_tag == tag_list:
                return None, -1  # Invalid Tag Found
        """
        LEN: consists of one or more consecutive bytes
        """
        offset += 1
        len_offset = 0
        if data_len <= offset:
            return None, -1  # Len is not available
        len_list = [data_list[offset]]
        if data_list[offset] & tag_mask_subsequent_bytes == tag_mask_subsequent_bytes:
            len_additional_bytes = data_list[offset] & len_mask_additional_bytes
            if data_len <= offset + len_mask_additional_bytes:
                return None, -1  # Mentioned Len is not available
            if len_additional_bytes == 0:
                return None, -1  # Not Multi Length, e.g.: F0
            while len_offset < len_additional_bytes:
                offset += 1
                len_offset += 1
                len_list.append(data_list[offset])
            len_offset = 0
            len_dec = PhUtil.hex_str_to_dec(
                PhUtil.to_hex_string(len_list[len_offset + 1: len_offset + 1 + len_additional_bytes],
                                     PhConstants.FORMAT_HEX_STRING_AS_PACK))
        else:
            len_dec = len_list[len_offset]
        """
        Value
        """
        offset += 1
        if data_len < offset + len_dec:
            return None, -1  # Value is not available
        value_list = data_list[offset:offset + len_dec]
        offset += len_dec
        return Tlv(tag_list, len_list, value_list, len_dec, level), offset

    def process_tlv(self, input_data_list=None, level=0, non_tlv_neighbor=None, info_data=None):
        if input_data_list is None:
            input_data_list = self.input_data_list
        if non_tlv_neighbor is None:
            non_tlv_neighbor = self.non_tlv_neighbor
        if info_data is None:
            info_data = self.info_data
        offset = 0
        tlv_data = []
        non_tlv_data = []
        sub_tlv_obj_temp = []
        while offset < len(input_data_list):
            tlv_obj, temp_offset = self.__process_tlv(input_data_list, level, offset)
            if tlv_obj is None:
                break
            offset = temp_offset
            if len(tlv_obj.value_list) > 2:  # Possible nested TLV, parse it & store result in temp list
                sub_tlv_data_temp = (
                    self.process_tlv(tlv_obj.value_list, level=level + 1, non_tlv_neighbor=False, info_data=info_data)
                    .get(PhKeys.RESULT_PROCESSED, None))
                # Always returned a list
                if isinstance(sub_tlv_data_temp[0], Tlv):  # Nested TLV Suspected, & Found
                    tlv_obj.value_list = sub_tlv_data_temp.copy()
                    sub_tlv_obj_temp.append(tlv_obj)
                else:
                    sub_tlv_obj_temp.append(tlv_obj)  # Nested TLV Suspected, but Not Found
            else:  # No Nested TLV
                # TODO: ESIM-1475: Actually this should be concat_tlv_obj_temp, For Concatenated TLV Handling
                sub_tlv_obj_temp.append(tlv_obj)
        count_sub_tlv_obj_temp = len(sub_tlv_obj_temp)
        # Check if at least one TLV is found
        if count_sub_tlv_obj_temp > 0:
            # Check if a neighborhood is perfect or not
            neighbour_tantrum = True if tlv_obj is None else False
            if neighbour_tantrum:
                # non TLV data
                if info_data is not None:
                    info_data.set_info(f'Non TLV data Found; non_tlv_neighbor is {non_tlv_neighbor}')
                non_tlv_data = input_data_list[offset:]
                if non_tlv_neighbor is False:
                    # no need to handle non_tlv_neighbor
                    return {PhKeys.RESULT_PROCESSED: input_data_list}
                tlv_obj = sub_tlv_obj_temp[-1]
            for item in sub_tlv_obj_temp:
                if isinstance(item, list):
                    tlv_obj.value_list = item
                else:  # instance of tlv
                    if item.level == tlv_obj.level:
                        # print('Same Level')
                        tlv_data.append(item)
                    else:
                        # print('Parent')
                        tlv_obj.value_list = item
                        tlv_data.append(tlv_obj)
            result = {PhKeys.RESULT_PROCESSED: tlv_data}
            if non_tlv_data:
                result.update({PhKeys.RESULT_UNPROCESSED: non_tlv_data})
            return result
        else:
            # No TLV Found
            return {PhKeys.RESULT_PROCESSED: input_data_list}



# -*- coding: UTF−8 -*-

import os
from setuptools import setup, find_packages

from tlv_play.main.helper.constants_config import ConfigConst

# all packages dependencies
packages = find_packages()
if not packages:
    print('Selecting Hardcoded Packages')
    packages = [
        ConfigConst.TOOL_PACKAGE_NAME,
        ConfigConst.TOOL_TEST_PACKAGE_NAME,
    ]
# Packages are ['tlv_play.main', 'tlv_play.test']
print(f'Packages are {packages}')

# run-time dependencies
install_reqs = [
    'playhelpers',
]

# build-time dependencies
setup_reqs = [
    'playhelpers',
]

# get long description from the README.md
with open(os.path.join(os.path.dirname(__file__), "README.md"), "r", encoding="utf-8") as fd:
    long_description = fd.read()

setup(
    use_incremental=True,
    setup_requires=setup_reqs,
    name=ConfigConst.TOOL_NAME,
    author="Pratik Jaiswal",
    author_email="impratikjaiswal@gmail.com",
    description=ConfigConst.TOOL_DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=ConfigConst.TOOL_URL,
    project_urls={
        "Bug Tracker": ConfigConst.TOOL_URL_BUG_TRACKER,
    },
    keywords=ConfigConst.TOOL_META_KEYWORDS,
    license="GNU GENERAL PUBLIC LICENSE v3.0",
    python_requires=">=3.9",
    packages=packages,
    install_requires=install_reqs,
    # test_suite="test.sample_package",
)



"""
Provides tlv_play version information.
"""

# This file is auto-generated! Do not edit!
# Use `incremental` to change this file.

from incremental import Version

__version__ = Version("tlv_play", 4, 5, 3)
__all__ = ["__version__"]



import subprocess
import traceback

import binascii
from play_helpers.ph_constants import PhConstants
from play_helpers.ph_data_master import PhMasterData, PhMasterDataKeys
from play_helpers.ph_exception_helper import PhExceptionHelper
from play_helpers.ph_keys import PhKeys
from play_helpers.ph_modes_error_handling import PhErrorHandlingModes
from play_helpers.ph_util import PhUtil

from tlv_play.main.convert import converter
from tlv_play.main.convert.converter import handle_web_request, set_defaults
from tlv_play.main.convert.parser import process_all_data_types
from tlv_play.main.helper.data import Data
from tlv_play.main.helper.infodata import InfoData
from tlv_play.main.helper.metadata import MetaData


class DataTypeMaster(object):
    def __init__(self):
        # Common Objects
        self.print_input = None
        self.print_output = None
        self.print_info = None
        self.quite_mode = None
        self.remarks = None
        self.encoding = None
        self.encoding_errors = None
        self.output_path = None
        self.output_file_name_keyword = None
        self.archive_output = None
        self.archive_output_format = None
        # Specific Objects
        self.length_in_decimal = None
        self.value_in_ascii = None
        self.one_liner = None
        self.non_tlv_neighbor = None
        self.data_pool = []
        self.__master_data = PhMasterData(
            data=Data(input_data=None),
            meta_data=MetaData(input_data_org=None),
            error_data=PhExceptionHelper(msg_key=None),
            info_data=InfoData(info=None)
        )

    def set_print_input(self, print_input):
        self.print_input = print_input

    def set_print_output(self, print_output):
        self.print_output = print_output

    def set_print_info(self, print_info):
        self.print_info = print_info

    def set_quiet_mode(self, quite_mode):
        self.quite_mode = quite_mode

    def set_remarks(self, remarks):
        self.remarks = remarks

    def set_encoding(self, encoding):
        self.encoding = encoding

    def set_encoding_errors(self, encoding_errors):
        self.encoding_errors = encoding_errors

    def set_output_path(self, output_path):
        self.output_path = output_path

    def set_output_file_name_keyword(self, output_file_name_keyword):
        self.output_file_name_keyword = output_file_name_keyword

    def set_archive_output(self, archive_output):
        self.archive_output = archive_output

    def set_archive_output_format(self, archive_output_format):
        self.archive_output_format = archive_output_format

    def set_length_in_decimal(self, length_in_decimal):
        self.length_in_decimal = length_in_decimal

    def set_value_in_ascii(self, value_in_ascii):
        self.value_in_ascii = value_in_ascii

    def set_one_liner(self, one_liner):
        self.one_liner = one_liner

    def set_non_tlv_neighbor(self, non_tlv_neighbor):
        self.non_tlv_neighbor = non_tlv_neighbor

    def set_data_pool(self, data_pool):
        self.data_pool = data_pool

    def process_safe(self, error_handling_mode, data=None):
        """

        :param data:
        :param error_handling_mode:
        :return:
        """
        if data is None:
            data = self.data_pool
        if isinstance(data, list):
            """
            Handle Requests Pool; Multiple Data Request are sent
            """
            for data_item in data:
                self.process_safe(error_handling_mode=error_handling_mode, data=data_item)
            return
        """
        Handle Individual Request
        """
        try:
            if isinstance(data, dict):
                """
                Web Form
                """
                data = handle_web_request(data)
            self.__process_safe_individual(data)
        except Exception as e:
            known = False
            summary_msg = None
            exception_object = e.args[0] if len(e.args) > 0 else e
            if not isinstance(exception_object, PhExceptionHelper):
                # for scenarios like FileExistsError where a touple is returned, (17, 'Cannot create a file when that file already exists')
                exception_object = PhExceptionHelper(exception=e)
            if isinstance(e, binascii.Error):
                known = True
                summary_msg = PhConstants.INVALID_INPUT_DATA
            elif isinstance(e, ValueError):
                known = True
            elif isinstance(e, PermissionError):
                known = True
                summary_msg = PhConstants.READ_WRITE_ERROR
            elif isinstance(e, FileExistsError):
                known = True
                summary_msg = PhConstants.WRITE_PATH_ERROR
            elif isinstance(e, subprocess.TimeoutExpired):
                known = True
                summary_msg = PhConstants.TIME_OUT_ERROR
            elif isinstance(e, subprocess.CalledProcessError):
                known = True
                summary_msg = e.stderr if e.stderr else PhConstants.NON_ZERO_EXIT_STATUS_ERROR
            exception_object.set_summary_msg(summary_msg)
            self.__master_data.set_master_data(PhMasterDataKeys.ERROR_DATA, exception_object)
            converter.print_data(master_data=self.__master_data)
            msg = PhConstants.SEPERATOR_TWO_WORDS.join(
                [PhConstants.KNOWN if known else PhConstants.UNKNOWN, exception_object.get_details()])
            print(f'{msg}')
            if not known:
                traceback.print_exc()
            if error_handling_mode == PhErrorHandlingModes.STOP_ON_ERROR:
                raise

    def __process_safe_individual(self, data):
        """
        Handle Individual Request
        :param data:
        :return:
        """
        if isinstance(data, Data):
            data.print_input = data.print_input if data.print_input is not None else self.print_input
            data.print_output = data.print_output if data.print_output is not None else self.print_output
            data.print_info = data.print_info if data.print_info is not None else self.print_info
            data.quite_mode = data.quite_mode if data.quite_mode is not None else self.quite_mode
            data.remarks = data.remarks if data.remarks is not None else self.remarks
            data.encoding = data.encoding if data.encoding is not None else self.encoding
            data.encoding_errors = data.encoding_errors if data.encoding_errors is not None else self.encoding_errors
            data.output_path = data.output_path if data.output_path is not None else self.output_path
            data.output_file_name_keyword = data.output_file_name_keyword if data.output_file_name_keyword is not None else self.output_file_name_keyword
            data.archive_output = data.archive_output if data.archive_output is not None else self.archive_output
            data.archive_output_format = data.archive_output_format if data.archive_output_format is not None else self.archive_output_format
            data.length_in_decimal = data.length_in_decimal if data.length_in_decimal is not None else self.length_in_decimal
            data.value_in_ascii = data.value_in_ascii if data.value_in_ascii is not None else self.value_in_ascii
            data.one_liner = data.one_liner if data.one_liner is not None else self.one_liner
            data.non_tlv_neighbor = data.non_tlv_neighbor if data.non_tlv_neighbor is not None else self.non_tlv_neighbor
        else:
            data = Data(
                input_data=data,
                print_input=self.print_input,
                print_output=self.print_output,
                print_info=self.print_info,
                quite_mode=self.quite_mode,
                remarks=self.remarks,
                encoding=self.encoding,
                encoding_errors=self.encoding_errors,
                output_path=self.output_path,
                output_file_name_keyword=self.output_file_name_keyword,
                archive_output=self.archive_output,
                archive_output_format=self.archive_output_format,
                length_in_decimal=self.length_in_decimal,
                value_in_ascii=self.value_in_ascii,
                one_liner=self.one_liner,
                non_tlv_neighbor=self.non_tlv_neighbor,
            )
        meta_data = MetaData(input_data_org=data.input_data)
        info_data = InfoData()
        self.__master_data = PhMasterData(data=data, meta_data=meta_data, error_data=None, info_data=info_data)
        process_all_data_types(data, meta_data, info_data)

    def get_output_data(self, only_output=True):
        """

        :return:
        """
        return self.__master_data.get_output_data(only_output=only_output)

    def to_dic(self, data):
        """

        :param data:
        :return:
        """
        set_defaults(data, None)
        common_data = {
            #
            PhKeys.INPUT_DATA: data.input_data,
            PhKeys.PRINT_INPUT: data.print_input,
            PhKeys.PRINT_OUTPUT: data.print_output,
            PhKeys.PRINT_INFO: data.print_info,
            PhKeys.QUITE_MODE: data.quite_mode,
            PhKeys.REMARKS: data.get_remarks_as_str(),
            PhKeys.ENCODING: data.encoding,
            PhKeys.ENCODING_ERRORS: data.encoding_errors,
            PhKeys.OUTPUT_PATH: data.output_path,
            PhKeys.OUTPUT_FILE_NAME_KEYWORD: data.output_file_name_keyword,
            PhKeys.ARCHIVE_OUTPUT: data.archive_output,
            PhKeys.ARCHIVE_OUTPUT_FORMAT: data.archive_output_format,
            #
            PhKeys.LENGTH_IN_DECIMAL: data.length_in_decimal,
            PhKeys.VALUE_IN_ASCII: data.value_in_ascii,
            PhKeys.ONE_LINER: data.one_liner,
            PhKeys.NON_TLV_NEIGHBOR: data.non_tlv_neighbor,
            #
            PhKeys.DATA_GROUP: data.data_group,
        }
        return PhUtil.dict_clean(common_data)



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_exception_helper import PhExceptionHelper

from tlv_play.main.tlv.tlv_handler import TlvHandler
from tlv_play.main.tlv.tlv_parser import TlvParser


def handle_data(data, meta_data, info_data, flip_output=False):
    """

    :param data:
    :param meta_data:
    :param info_data:
    :param flip_output:
    :return:
    """
    input_data = data.input_data
    # input_File_path = meta_data.input_data_org if meta_data.input_mode_key == PhKeys.INPUT_FILE else None
    # input_format = data.input_format
    # output_format = data.output_format
    # if flip_output is True:
    #     input_data = meta_data.parsed_data
    #     input_format = data.output_format
    #     output_format = data.input_format
    # parse_only = True
    # asn1_element = data.asn1_element
    if not data.input_data:
        raise ValueError(PhExceptionHelper(msg_key=PhConstants.MISSING_INPUT_DATA))
    res = __handle_data(input_data=input_data, length_in_decimal=data.length_in_decimal,
                        value_in_ascii=data.value_in_ascii, one_liner=data.one_liner,
                        non_tlv_neighbor=data.non_tlv_neighbor, info_data=info_data)
    if flip_output is True:
        meta_data.re_parsed_data = res
    else:
        meta_data.parsed_data = res


def __handle_data(input_data, length_in_decimal, value_in_ascii, one_liner, non_tlv_neighbor, info_data):
    """

    :param input_data:
    :param length_in_decimal:
    :param value_in_ascii:
    :param one_liner:
    :param non_tlv_neighbor:
    :param info_data:
    :return:
    """
    tlv_handler_result = TlvHandler(input_data=input_data, non_tlv_neighbor=non_tlv_neighbor,
                                    info_data=info_data).process_tlv()
    output_data = TlvParser(tlv_handler_result=tlv_handler_result).get_printable_tlv(length_in_decimal, value_in_ascii,
                                                                                     one_liner)
    return output_data



import sys
from play_helpers.ph_modes_error_handling import PhErrorHandlingModes
from play_helpers.ph_modes_execution import PhExecutionModes
from play_helpers.ph_time import PhTime
from play_helpers.ph_util import PhUtil

from tlv_play.main.convert.converter import handle_web_request
from tlv_play.main.data_type.data_type_master import DataTypeMaster
from tlv_play.main.data_type.dev import Dev
from tlv_play.main.data_type.known_issues import KnownIssues
from tlv_play.main.data_type.sample import Sample
from tlv_play.main.data_type.unit_testing import UnitTesting
from tlv_play.main.data_type.user_data import UserData
from tlv_play.main.helper.constants_config import ConfigConst
from tlv_play.main.helper.defaults import Defaults
from tlv_play.test.test import Test

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

"""
Global Variables
"""
data_cli = None
execution_mode = None
error_handling_mode = None


def process_data():
    """

    :return:
    """
    global error_handling_mode
    data_type_user = [
        #####
        # Empty class for user usage
        ####
        UserData(),
    ]
    data_type_dev = [
        #####
        # class for dev
        #####
        Dev(),
    ]
    data_type_known_issues = [
        #####
        # class for known issues
        #####
        KnownIssues(),
    ]
    data_types_sample_generic = [
        #####
        # Sample With Plenty vivid Examples; Single as well as Bulk
        #####
    ]
    data_types_samples = [
        #####
        # Sample With Plenty vivid Examples; Single as well as Bulk
        #####
        Sample(),
    ]
    data_types_sample_specific = [
    ]
    data_type_unit_testing = [
        #####
        # Unit Testing
        #####
        UnitTesting(),
    ]
    data_type_unit_testing_external = [
        #####
        # Unit Testing External
        #####
        Test(),
    ]

    data_types_pool = {
        PhExecutionModes.USER: data_type_user,
        PhExecutionModes.SAMPLES_LIST: data_types_samples,
        PhExecutionModes.SAMPLE_GENERIC: data_types_sample_generic,
        PhExecutionModes.SAMPLE_SPECIFIC: data_types_sample_specific,
        PhExecutionModes.UNIT_TESTING: data_type_unit_testing,
        PhExecutionModes.UNIT_TESTING_EXTERNAL: data_type_unit_testing_external,
        PhExecutionModes.DEV: data_type_dev,
        PhExecutionModes.KNOWN_ISSUES: data_type_known_issues,
        PhExecutionModes.ALL: data_type_user
                              + data_types_samples
                              + data_types_sample_generic
                              + data_types_sample_specific
                              + data_type_unit_testing
                              + data_type_unit_testing_external
        # + data_type_dev
        # + data_type_known_issues
        ,
    }
    data_types = data_types_pool.get(execution_mode, Defaults.EXECUTION_MODE)
    if data_cli:
        _data_type = DataTypeMaster()
        _data_type.set_data_pool(data_pool=[data_cli])
        data_types = [_data_type]
    for data_type in data_types:
        PhUtil.print_heading(str_heading=f'Data Class: {str(data_type.__class__.__name__)}')
        if isinstance(data_type, UnitTesting):
            error_handling_mode = PhErrorHandlingModes.CONTINUE_ON_ERROR
        if isinstance(data_type, Dev):
            error_handling_mode = PhErrorHandlingModes.STOP_ON_ERROR
        if isinstance(data_type, Test):
            Test.test_all()
            continue
        if isinstance(data_type, Sample):
            # Validate & Print Sample Data For Web
            PhUtil.print_iter(Sample().get_sample_data_pool_for_web(), header='Sample Data')
        if not data_cli:
            data_type.set_print_input()
            data_type.set_print_output()
            data_type.set_print_info()
            data_type.set_quiet_mode()
            data_type.set_remarks()
            data_type.set_encoding()
            data_type.set_encoding_errors()
            data_type.set_output_path()
            data_type.set_output_file_name_keyword()
            data_type.set_archive_output()
            data_type.set_archive_output_format()
            #
            data_type.set_one_liner()
            data_type.set_value_in_ascii()
            data_type.set_length_in_decimal()
            data_type.set_non_tlv_neighbor()
            #
            data_type.set_data_pool()
        DataTypeMaster.process_safe(data_type, error_handling_mode)


def handle_cli_request(**kwargs):
    """

    :param kwargs:
    :return:
    """
    global data_cli
    data_cli = handle_web_request(kwargs)


def print_configurations():
    # Print Versions
    version_parameters_pool = [
        {'tool_name': ConfigConst.TOOL_NAME, 'tool_version': ConfigConst.TOOL_VERSION},
    ]
    PhUtil.print_version(parameters_pool=version_parameters_pool)


def main():
    """

    :return:
    """
    """
    Time Object
    """
    ph_time_obj = PhTime()
    ph_time_obj.start()
    """
    Handle Args
    """
    if len(sys.argv) > 1:
        standalone_mode = False
        # callback is not received for '--help', so handle differently
        if sys.argv[1] == '--help':
            # Print Configurations
            print_configurations()
            standalone_mode = True
        handle_cli_request(standalone_mode=standalone_mode)
    """
    Configurations
    """
    # Do Configurations, as per your Need
    set_configurations()
    # Print Configurations
    print_configurations()
    """
    Process
    """
    process_data()
    """
    Wrap up, All Done
    """
    ph_time_obj.stop()
    ph_time_obj.print()
    PhUtil.print_done()


def set_configurations():
    """

    :return:
    """
    global execution_mode, error_handling_mode
    """
    Set Execution Mode, First time users can try #PhExecutionModes.SAMPLE_GENERIC
    """
    execution_mode = PhExecutionModes.USER
    error_handling_mode = PhErrorHandlingModes.CONTINUE_ON_ERROR


if __name__ == '__main__':
    main()
